open Attribute_value
let attribute_parsers =
  [("a_aria", (wrap (spaces string)));
  ("a_role", (wrap (spaces string)));
  ("a_property", (wrap string));
  ("a_style", (wrap string));
  ("a_media",
    (wrap (commas (total_variant Html_types_reflected.mediadesc_token))));
  ("a_defer", (nowrap unit));
  ("a_http_equiv", (wrap string));
  ("a_content", (wrap string));
  ("a_target", (wrap string));
  ("a_scrolling", (wrap variant));
  ("a_marginwidth", (wrap int));
  ("a_marginheight", (wrap int));
  ("a_frameborder", (wrap variant));
  ("a_codetype", (wrap string));
  ("a_data", (wrap string));
  ("a_usemap", (wrap string));
  ("a_coords", (wrap (commas int)));
  ("a_shape", (wrap variant));
  ("a_width", (wrap int));
  ("a_height", (wrap int));
  ("a_alt", (wrap string));
  ("a_char", (wrap char));
  ("a_rules", (wrap variant));
  ("a_border", (wrap int));
  ("a_summary", (wrap string));
  ("a_scope", (wrap variant));
  ("a_rowspan", (wrap int));
  ("a_headers", (wrap (spaces string)));
  ("a_colspan", (wrap int));
  ("a_axis", (wrap string));
  ("a_align", (wrap variant));
  ("a_label", (wrap string));
  ("a_menu_type", (wrap variant));
  ("a_command_type", (wrap variant));
  ("a_script_type", (wrap script_type));
  ("a_button_type", (wrap variant));
  ("a_readonly", (nowrap unit));
  ("a_disabled", (nowrap unit));
  ("a_float_value", (wrap float));
  ("a_value", (wrap string));
  ("a_int_value", (wrap int));
  ("a_text_value", (wrap string));
  ("a_input_type", (wrap variant));
  ("a_src", (wrap string));
  ("a_size", (wrap int));
  ("a_selected", (nowrap unit));
  ("a_rows", (wrap int));
  ("a_name", (wrap string));
  ("a_multiple", (nowrap unit));
  ("a_formmethod", (wrap variant));
  ("a_method", (wrap variant));
  ("a_minlength", (wrap int));
  ("a_maxlength", (wrap int));
  ("a_for_list", (wrap (spaces string)));
  ("a_output_for", (wrap (spaces string)));
  ("a_for", (wrap string));
  ("a_label_for", (wrap string));
  ("a_enctype", (wrap string));
  ("a_cols", (wrap int));
  ("a_checked", (nowrap unit));
  ("a_action", (wrap string));
  ("a_datetime", (wrap string));
  ("a_mime_type", (wrap string));
  ("a_tabindex", (wrap int));
  ("a_rel", (wrap (spaces (total_variant Html_types_reflected.linktype))));
  ("a_download", (wrap (option "" string)));
  ("a_hreflang", (wrap string));
  ("a_href", (wrap string));
  ("a_accept", (wrap (commas string)));
  ("a_accept_charset", (wrap (spaces string)));
  ("a_charset", (wrap string));
  ("a_accesskey", (wrap char));
  ("a_xml_space", (wrap variant));
  ("a_cite", (wrap string));
  ("a_manifest", (wrap string));
  ("a_xmlns", (wrap xmlns));
  ("a_version", (wrap string));
  ("a_wrap", (wrap variant));
  ("a_translate", (wrap variant));
  ("a_step", (wrap (option "any" float)));
  ("a_start", (wrap int));
  ("a_img_sizes", (wrap (commas string)));
  ("a_srcset", (wrap (commas srcset_element)));
  ("a_srclang", (wrap string));
  ("a_span", (wrap int));
  ("a_sizes", (wrap (option "any" (spaces icon_size))));
  ("a_seamless", (nowrap unit));
  ("a_scoped", (nowrap unit));
  ("a_spellcheck", (wrap bool));
  ("a_sandbox", (wrap sandbox));
  ("a_reversed", (nowrap unit));
  ("a_required", (nowrap unit));
  ("a_referrerpolicy", (wrap (variant_or_empty "Empty")));
  ("a_radiogroup", (wrap string));
  ("a_pubdate", (nowrap unit));
  ("a_preload", (wrap variant));
  ("a_poster", (wrap string));
  ("a_placeholder", (wrap string));
  ("a_pattern", (wrap string));
  ("a_optimum", (wrap float));
  ("a_open", (nowrap unit));
  ("a_novalidate", (nowrap unit));
  ("a_inputmode", (wrap variant));
  ("a_input_min", (wrap number_or_datetime));
  ("a_min", (wrap float));
  ("a_input_max", (wrap number_or_datetime));
  ("a_max", (wrap float));
  ("a_low", (wrap float));
  ("a_loop", (nowrap unit));
  ("a_list", (wrap string));
  ("a_keytype", (wrap string));
  ("a_ismap", (nowrap unit));
  ("a_icon", (wrap string));
  ("a_high", (wrap float));
  ("a_hidden", (nowrap unit));
  ("a_formtarget", (wrap string));
  ("a_formnovalidate", (nowrap unit));
  ("a_formenctype", (wrap string));
  ("a_formaction", (wrap string));
  ("a_form", (wrap string));
  ("a_draggable", (wrap bool));
  ("a_dir", (wrap variant));
  ("a_controls", (nowrap unit));
  ("a_contextmenu", (wrap string));
  ("a_contenteditable", (wrap bool));
  ("a_challenge", (wrap string));
  ("a_mediagroup", (wrap string));
  ("a_integrity", (wrap string));
  ("a_crossorigin", (wrap variant));
  ("a_muted", (nowrap unit));
  ("a_autoplay", (nowrap unit));
  ("a_autofocus", (nowrap unit));
  ("a_async", (nowrap unit));
  ("a_autocomplete", (wrap autocomplete));
  ("a_allowpaymentrequest", (nowrap unit));
  ("a_allowfullscreen", (nowrap unit));
  ("a_onkeyup", (nowrap string));
  ("a_onkeydown", (nowrap string));
  ("a_onkeypress", (nowrap string));
  ("a_ontouchcancel", (nowrap string));
  ("a_ontouchmove", (nowrap string));
  ("a_ontouchend", (nowrap string));
  ("a_ontouchstart", (nowrap string));
  ("a_onmouseout", (nowrap string));
  ("a_onmousemove", (nowrap string));
  ("a_onmouseover", (nowrap string));
  ("a_onmouseup", (nowrap string));
  ("a_onmousedown", (nowrap string));
  ("a_ondrop", (nowrap string));
  ("a_ondragstart", (nowrap string));
  ("a_ondragover", (nowrap string));
  ("a_ondragleave", (nowrap string));
  ("a_ondragenter", (nowrap string));
  ("a_ondragend", (nowrap string));
  ("a_ondrag", (nowrap string));
  ("a_ondblclick", (nowrap string));
  ("a_oncontextmenu", (nowrap string));
  ("a_onclick", (nowrap string));
  ("a_onmessage", (nowrap string));
  ("a_onloadstart", (nowrap string));
  ("a_onloadedmetadata", (nowrap string));
  ("a_onloadeddata", (nowrap string));
  ("a_onload", (nowrap string));
  ("a_onwaiting", (nowrap string));
  ("a_onvolumechange", (nowrap string));
  ("a_onunload", (nowrap string));
  ("a_onundo", (nowrap string));
  ("a_ontimeupdate", (nowrap string));
  ("a_onsuspend", (nowrap string));
  ("a_onsubmit", (nowrap string));
  ("a_onstorage", (nowrap string));
  ("a_onstalled", (nowrap string));
  ("a_onshow", (nowrap string));
  ("a_onselect", (nowrap string));
  ("a_onseeking", (nowrap string));
  ("a_onseeked", (nowrap string));
  ("a_onscroll", (nowrap string));
  ("a_onresize", (nowrap string));
  ("a_onredo", (nowrap string));
  ("a_onreadystatechange", (nowrap string));
  ("a_onratechange", (nowrap string));
  ("a_onprogress", (nowrap string));
  ("a_onpopstate", (nowrap string));
  ("a_onpageshow", (nowrap string));
  ("a_onpagehide", (nowrap string));
  ("a_onplaying", (nowrap string));
  ("a_onplay", (nowrap string));
  ("a_onpause", (nowrap string));
  ("a_ononline", (nowrap string));
  ("a_onoffline", (nowrap string));
  ("a_onmousewheel", (nowrap string));
  ("a_oninvalid", (nowrap string));
  ("a_oninput", (nowrap string));
  ("a_onhashchange", (nowrap string));
  ("a_onforminput", (nowrap string));
  ("a_onformchange", (nowrap string));
  ("a_onfocus", (nowrap string));
  ("a_onerror", (nowrap string));
  ("a_onended", (nowrap string));
  ("a_onemptied", (nowrap string));
  ("a_ondurationchange", (nowrap string));
  ("a_onclose", (nowrap string));
  ("a_onchange", (nowrap string));
  ("a_oncanplaythrough", (nowrap string));
  ("a_oncanplay", (nowrap string));
  ("a_onblur", (nowrap string));
  ("a_onbeforeunload", (nowrap string));
  ("a_onbeforeprint", (nowrap string));
  ("a_onafterprint", (nowrap string));
  ("a_onabort", (nowrap string));
  ("a_lang", (wrap string));
  ("a_xml_lang", (wrap string));
  ("a_title", (wrap string));
  ("a_id", (wrap string));
  ("a_user_data", (wrap string));
  ("a_class", (wrap (spaces string)))]
let renamed_attributes =
  [("a_menu_type", "type", ["menu"]);
  ("a_command_type", "type", ["command"]);
  ("a_script_type", "type", ["script"]);
  ("a_button_type", "type", ["button"]);
  ("a_float_value", "value", ["meter"; "progress"]);
  ("a_int_value", "value", ["li"]);
  ("a_text_value", "value", ["option"; "button"; "param"]);
  ("a_input_type", "type", ["input"]);
  ("a_output_for", "for", ["output"]);
  ("a_label_for", "for", ["label"]);
  ("a_mime_type", "type", ["source"; "link"; "area"; "embed"; "object"]);
  ("a_img_sizes", "sizes", ["img"]);
  ("a_input_min", "min", ["input"]);
  ("a_input_max", "max", ["input"])]
let labeled_attributes =
  [("link", "rel",
     (wrap (spaces (total_variant Html_types_reflected.linktype))));
  ("link", "href", (wrap string));
  ("command", "label", (wrap string));
  ("optgroup", "label", (wrap string));
  ("area", "alt", (wrap string));
  ("video", "src", (wrap string));
  ("audio", "src", (wrap string));
  ("img", "src", (wrap string));
  ("img", "alt", (wrap string));
  ("bdo", "dir", (wrap variant))]
open Element_content
let element_assemblers =
  [("ruby", star);
  ("rp", star);
  ("rt", star);
  ("link", nullary);
  ("style", star);
  ("meta", nullary);
  ("template", star);
  ("noscript", star);
  ("script", script);
  ("menu", menu);
  ("command", nullary);
  ("summary", star);
  ("details", details);
  ("output_elt", star);
  ("meter", star);
  ("progress", star);
  ("keygen", nullary);
  ("textarea", textarea);
  ("option", unary);
  ("optgroup", star);
  ("datalist", (comp_filter_whitespace datalist));
  ("select", (comp_filter_whitespace star));
  ("button", star);
  ("input", nullary);
  ("label", star);
  ("legend", star);
  ("fieldset", fieldset);
  ("form", star);
  ("tr", (comp_filter_whitespace star));
  ("th", star);
  ("td", star);
  ("tfoot", (comp_filter_whitespace star));
  ("tbody", (comp_filter_whitespace star));
  ("thead", (comp_filter_whitespace star));
  ("col", nullary);
  ("colgroup", (comp_filter_whitespace star));
  ("tablex", (comp_filter_whitespace table));
  ("table", (comp_filter_whitespace table));
  ("caption", star);
  ("map", star);
  ("area", nullary);
  ("source", nullary);
  ("canvas", star);
  ("video", audio_video);
  ("audio", audio_video);
  ("embed", nullary);
  ("param", nullary);
  ("object_", object_);
  ("iframe", star);
  ("picture", (comp_filter_whitespace picture));
  ("img", nullary);
  ("ins", star);
  ("del", star);
  ("a", star);
  ("var", star);
  ("time", star);
  ("strong", star);
  ("span", star);
  ("samp", star);
  ("q", star);
  ("kbd", star);
  ("em", star);
  ("dfn", star);
  ("code", star);
  ("cite", star);
  ("br", nullary);
  ("abbr", star);
  ("bdo", star);
  ("wbr", nullary);
  ("mark", star);
  ("sup", star);
  ("sub", star);
  ("small", star);
  ("u", star);
  ("i", star);
  ("b", star);
  ("hr", nullary);
  ("figure", figure);
  ("figcaption", star);
  ("li", star);
  ("dt", star);
  ("dd", star);
  ("ul", (comp_filter_whitespace star));
  ("ol", (comp_filter_whitespace star));
  ("dl", star);
  ("div", star);
  ("dialog", star);
  ("blockquote", star);
  ("pre", star);
  ("p", star);
  ("main", star);
  ("aside", star);
  ("article", star);
  ("address", star);
  ("hgroup", star);
  ("h6", star);
  ("h5", star);
  ("h4", star);
  ("h3", star);
  ("h2", star);
  ("h1", star);
  ("nav", star);
  ("section", star);
  ("header", star);
  ("footer", star);
  ("body", star);
  ("title", unary);
  ("base", nullary);
  ("head", (comp_filter_whitespace head));
  ("html", (comp_filter_whitespace html))]
let renamed_elements =
  [("output", "output_elt"); ("table", "tablex"); ("object", "object_")]
